/* Taken and adapted from the tetris plugin,
   (c) Clemens Kirchgatterer <clemens@thf.ath.cx> 2002   
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <vdr/config.h>
#include "display.h"

#include "setup.h"
#include "colormapping.h"

namespace Display {

static Mode displayMode=Full;
static cOsd *osd=0;
static const cFont *font=0;

static int bpp=0, messageWidth=0;
static tColor currentBackground=clrGray50;


int Open( Mode mode) {
   if (osd) 
      return -1;
      
   displayMode=mode;
   int height=ttSetup.OSDheight;
   int width=ttSetup.OSDwidth;
   messageWidth=0;
   int x0=Setup.OSDLeft+(Setup.OSDWidth-ttSetup.OSDwidth);
   int y0=Setup.OSDTop +(Setup.OSDHeight-ttSetup.OSDheight);
   
   if (mode == HalfLower || mode== HalfUpper) {
      height /= 2;
      height -= (height%4);
      y0 += height;
   }
   
   osd = cOsdProvider::NewOsd(x0, y0);
   tArea Areas[] = { { 0, 0, width - 1, height - 1, 4 } };
   if (osd->CanHandleAreas(Areas, sizeof(Areas) / sizeof(tArea)) == oeOk) {
      osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
      bpp=4;
   } else {
      tArea Areas[] = { { 0, 0, width - 1, height - 1, 2 } };
      osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
      bpp=2;
   }
   font=cFont::GetFont(fontSml);
   
   return (0);
}


int
Close(void) {
   if (!osd) 
      return -1;
   delete osd;
   osd=0;
   return 0;
}

int  CheckMode(Display::Mode mode) {
   if (osd && displayMode != mode) {
      if ( (displayMode==Full || displayMode==ZoomUpper || displayMode==ZoomLower) 
              && (mode==HalfUpper || mode==HalfLower) 
              || (mode==Full || mode==ZoomUpper || mode==ZoomLower) 
              && (displayMode==HalfUpper || displayMode==HalfLower)  
         ) 
      {
         Display::Close();
         return Display::Open(mode);
      } else {
         displayMode=mode; //no need to reopen if size does not change
         return 0;
      }
   } else if (!osd)
      return Display::Open(mode);
   else
      return 0;
}

int  Bpp() {
   return bpp;
}

int
Width(void) {
   return (osd) ? osd->Width() : 0;
}

int
Height(void) {
   return (osd) ? osd->Height() : 0;
}

int
FontHeight(void) {
   //This implementation uses its own teletext font with no relation to VDR's cFont class
   return (24);
}

tColor GetColor(TeletextColor color, TeletextColorType type) {
   if (Bpp() == 2) {
      switch ((type==ColorTypeNormal) ? color : color+8) {
         case 0: return COLORMAPPING_BLACK_2;
         case 1: return COLORMAPPING_RED_2;
         case 2: return COLORMAPPING_GREEN_2;
         case 3: return COLORMAPPING_YELLOW_2;
         case 4: return COLORMAPPING_BLUE_2;
         case 5: return COLORMAPPING_MAGENTA_2;
         case 6: return COLORMAPPING_CYAN_2;
         case 7: return COLORMAPPING_WHITE_2;
         
         case 8:  return COLORMAPPING_INVERTEDBLACK_2;
         case 9:  return COLORMAPPING_INVERTEDRED_2;
         case 10: return COLORMAPPING_INVERTEDGREEN_2;
         case 11: return COLORMAPPING_INVERTEDYELLOW_2;
         case 12: return COLORMAPPING_INVERTEDBLUE_2;
         case 13: return COLORMAPPING_INVERTEDMAGENTA_2;
         case 14: return COLORMAPPING_INVERTEDCYAN_2;
         case 15: return COLORMAPPING_INVERTEDWHITE_2;
         default: return COLORMAPPING_BLACK_2;
      }
   } else {
      switch ((type==ColorTypeNormal) ? color : color+8) {
         case 0:  return COLORMAPPING_BLACK_3;
         case 1:  return COLORMAPPING_RED_3;
         case 2:  return COLORMAPPING_GREEN_3;
         case 3:  return COLORMAPPING_YELLOW_3;
         case 4:  return COLORMAPPING_BLUE_3;
         case 5:  return COLORMAPPING_MAGENTA_3;
         case 6:  return COLORMAPPING_CYAN_3;
         case 7:  return COLORMAPPING_WHITE_3;
         
         case 8:  return COLORMAPPING_INVERTEDBLACK_3;
         case 9:  return COLORMAPPING_INVERTEDRED_3;
         case 10: return COLORMAPPING_INVERTEDGREEN_3;
         case 11: return COLORMAPPING_INVERTEDYELLOW_3;
         case 12: return COLORMAPPING_INVERTEDBLUE_3;
         case 13: return COLORMAPPING_INVERTEDMAGENTA_3;
         case 14: return COLORMAPPING_INVERTEDCYAN_3;
         case 15: return COLORMAPPING_INVERTEDWHITE_3;
         default: return COLORMAPPING_BLACK_3;
      }
   }
}

//necessary when scaling/copying: 
//If the indexed order is different between bitmaps, colors will be exchanged
void SetBitmapPalette(cBitmap &bitmap) {
   switch (bitmap.Bpp()) {
   case 1:
      bitmap.Index(COLORMAPPING_BLACK_2);
      bitmap.Index(COLORMAPPING_WHITE_2);
   case 2:
      bitmap.Index(COLORMAPPING_2_COLOR1);
      bitmap.Index(COLORMAPPING_2_COLOR2);
      bitmap.Index(COLORMAPPING_2_COLOR3);
      bitmap.Index(COLORMAPPING_2_COLOR4);
      break;
   case 4:
      bitmap.Index(COLORMAPPING_3_COLOR1);
      bitmap.Index(COLORMAPPING_3_COLOR2);
      bitmap.Index(COLORMAPPING_3_COLOR3);
      bitmap.Index(COLORMAPPING_3_COLOR4);
      bitmap.Index(COLORMAPPING_3_COLOR5);
      bitmap.Index(COLORMAPPING_3_COLOR6);
      bitmap.Index(COLORMAPPING_3_COLOR7);
      bitmap.Index(COLORMAPPING_3_COLOR8);
      break;
   }
}

void DrawBitmap(int x, int y, const cBitmap &bitmap) {
   if (osd) {
      osd->DrawBitmap(x, y, bitmap);
   }
}

void Message(const char *txt) {
   if (osd && txt) {
      messageWidth=font->Width(txt);
      osd->DrawText(0,0,txt, GetColor(ColorWhite), GetColor(ColorBlack), font);
   }
}

void ClearMessage() {
   if (osd)
      osd->DrawRectangle(0, 0, messageWidth, font->Height(), configurableBackground);
}

void
Update() {
   if (osd) {
      osd->Flush();
   }
}

tColor GetBackground() {
   return currentBackground;
}

void SetBackground(tColor bg) {
   currentBackground=bg;
}


} //end of namespace Display


