/***************************************************************************
 *       Copyright (c) 2003,2004 by Marcel Wiesweg                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>

#include "txtbitmap.h"
#include "menu.h"
#include "txtfont.h"
#include "setup.h"

cTxtBitmap::cTxtBitmap(int Width, int Height)
:cBitmap(Width, Height, Display::Bpp())
{
   origFg=(Display::TeletextColor)-1;
   origBg=(Display::TeletextColor)-1;
   Display::SetBitmapPalette(*this);
}

cTxtBitmap::~cTxtBitmap() {
}


void cTxtBitmap::Display(Display::Mode mode, int Offset, tColor, tColor) {
   int y0=0;
   int h=Height();
   if (mode==Display::HalfUpper || mode==Display::ZoomUpper || mode==Display::HalfLower || mode==Display::ZoomLower) {
      h /= 2;
   }
   if (mode==Display::HalfLower || mode==Display::ZoomLower)
      y0=Width()/2;
      
   cBitmap *bitmap=ScaleBitmap(this, Bpp(),  0, y0,   Width(), h,   Display::Width(), Display::Height());
   Display::DrawBitmap(0, Offset, *bitmap);
   Display::Update();
   delete bitmap;
}

void cTxtBitmap::DisplayClock() {
   cBitmap clockBitmap(8*12+4, Display::FontHeight(), Display::Bpp(), false);
   GenClockBitmap(&clockBitmap);
   
   float ratioW= ( (float)Display::Width() )/( (float)Width() );
   float ratioH= ( (float)Display::Height() )/( (float)Height() );
   int clockWidth= (int)( ((float)clockBitmap.Width()) * ratioW );
   int clockHeight=(int)( ((float)clockBitmap.Height()) * ratioH );
   cBitmap *bitmap=ScaleBitmap(&clockBitmap, Display::Bpp(), 0, 0, clockBitmap.Width(), clockBitmap.Height(),
                               clockWidth, clockHeight);
   Display::DrawBitmap(Display::Width()-bitmap->Width(), 0, *bitmap);
   Display::Update();
   delete bitmap;
}


//returns a new bitmap created on the heap with dimensions wneu x hneu, 
//and fills it with a scaled part of oldbitmap, starting at (x0|y0), width walt, height halt.
//Uses a very simple scaling algorithm. 
cBitmap *cTxtBitmap::ScaleBitmap(cBitmap* oldBitmap, int depth, int x0, int y0, int walt, int halt, int wneu, int hneu) {
   int W=oldBitmap->Width();
   int H=oldBitmap->Height();
   if (x0+walt>W) //parameters wrong, cut back to picture size
      walt=W-x0;
   if (y0+halt>H)
      halt=H-y0;
   if (halt==0 || walt==0)
      return 0;
   
   cBitmap *bitmap=new cBitmap(wneu, hneu, depth);
   Display::SetBitmapPalette(*bitmap);
   
   int/*float*/ oldX=0, oldY=0;
   for (float x=0;x<wneu;x++)
      for (float y=0;y<hneu;y++) {
         //mathematically: Index_neu(x,y) = Index_alt(x0+ratioW*x, y0+ratioH*y)
         //oldX=ratioW*x;
         //oldY=ratioH*y;
         oldX=(int)(((walt*10000/wneu)*x/10000));
         oldY=(int)(((halt*10000/hneu)*y/10000));
         bitmap->SetIndex((int)x,(int)y,*(oldBitmap->Data(oldX+x0, oldY+y0)) );
      }
   return bitmap;
         
}

void cTxtBitmap::SetLanguage(int lang)
{
   Language=lang;
}

unsigned char cTxtBitmap::LangChar(unsigned char Eingabe)
{
   unsigned char Rueckgabe;
   Rueckgabe=Eingabe;
   return Rueckgabe;
}

void cTxtBitmap::SetFGColor(Display::TeletextColor Color)
{
   origFg=Color;
   fgColor=Display::GetColor(Color, Display::ColorTypeNormal);
   if ( (fgColor == bgColor) && (origBg != origFg)) {
      fgColor=Display::GetColor(Color, Display::ColorTypeInverted);
   }
}

void cTxtBitmap::SetBGColor(Display::TeletextColor Color)
{
   origBg=Color;
   bgColor=Display::GetColor(Color, Display::ColorTypeNormal);
}

void cTxtBitmap::ExchangeColor(void)
{
   Display::TeletextColor oldBgColor=origBg;
   SetBGColor(origFg);
   SetFGColor(oldBgColor);
}

void cTxtBitmap::SetChar(int x, int y, unsigned char zeich, bool gr_mode, bool double_mode, bool mask_mode)
{
   unsigned int myfont[10];
   unsigned int Ergebnis;
   
   unsigned int used_zeich;
   int multiplikator;
   tColor farbe;
   
   used_zeich=LangChar(zeich);
   
   if (gr_mode==true) {
   if ((zeich>=0x20)&&(zeich<=0x3F)) used_zeich=used_zeich+0x80;
   if ((zeich>=0x60)&&(zeich<=0x7F)) used_zeich=used_zeich+0xA0;
   }
   
   //kopiere mir den Font heraus
   //char str[2]; str[0]=(char)used_zeich; str[1]=0;
   //printf(str);
   used_zeich=used_zeich-0x20;
   
   for (int i=0;i<10;i++)
   {
      myfont[i]=TXT_Font[used_zeich][i];
      if (mask_mode==true)
      myfont[i]=myfont[i]&TXT_Mask[i];
   }
   
   if (double_mode==true)
      multiplikator=4;
   else
      multiplikator=2;
   
   int txt_x_offset=0;
   int txt_y_offset=0;
   
   for (int yzeich=0;yzeich<10;yzeich++)
   {
      for (int xzeich=0;xzeich<12;xzeich++) {
         Ergebnis=(myfont[yzeich]<<xzeich)&0x8000;
         if (Ergebnis>0)
            farbe=fgColor;
         else
            farbe=bgColor;
      
         for (int i=0;i<multiplikator;i++) {
            DrawPixel(txt_x_offset+(x*12)+xzeich,txt_y_offset+(y*10*2)+(yzeich*multiplikator)+i,farbe);
         }
      }
   }

}


void cTxtBitmap::GenClockBitmap(cBitmap * bitmap)
{
   char Anzeige;
   int i=0;
   unsigned int Ergebnis;
   unsigned int myfont[10];
   tColor color;
   
   bitmap->Reset();
   Display::SetBitmapPalette(*bitmap); 
   
   char zeichen[9];
   time_t t=time(0);
   struct tm loct;
   localtime_r(&t, &loct);
   sprintf(zeichen, "%02d:%02d:%02d", loct.tm_hour, loct.tm_min, loct.tm_sec);
   
   // paste them into the bitmap
   while ((Anzeige = zeichen[i])!=0x00)
   {
      for (int k=0;k<10;k++)
         myfont[k]=TXT_Font[Anzeige-0x20][k];
   
      for (int yzeich=0;yzeich<10;yzeich++)
      {
         for (int xzeich=0;xzeich<12;xzeich++)
         {
            Ergebnis=(myfont[yzeich]<<xzeich)&0x8000;
            if (Ergebnis>0)
               color=Display::GetColor(Display::ColorWhite, Display::ColorTypeNormal);
            else
               color=Display::GetColor(Display::ColorBlack, Display::ColorTypeNormal);
            
            bitmap->DrawPixel(xzeich+i*12,yzeich*2  ,color);
            bitmap->DrawPixel(xzeich+i*12,yzeich*2+1,color);
         }
      }
      i++;
   }
}

